package main

import (
	"fmt"
	"os"

	"github.com/spf13/cobra"
	"github.com/spf13/viper"
)

var (
	cfgFile string
	dbPath  string
	addr    string

	rootCmd = &cobra.Command{
		Use:   "mcias",
		Short: "MCIAS - Metacircular Identity and Access System",
		Long: `MCIAS is the metacircular identity and access system, 
providing identity and authentication across metacircular projects.

It currently provides the following across metacircular services:
1. User password authentication
2. User token authentication
3. Database credential authentication`,
	}
)

func Execute() error {
	return rootCmd.Execute()
}

func init() {
	cobra.OnInitialize(initConfig)

	rootCmd.PersistentFlags().StringVar(&cfgFile, "config", "", "config file (default is $HOME/.mcias.yaml)")
	rootCmd.PersistentFlags().StringVar(&dbPath, "db", "mcias.db", "Path to SQLite database file")
	rootCmd.PersistentFlags().StringVar(&addr, "addr", ":8080", "Address to listen on")
	if err := viper.BindPFlag("db", rootCmd.PersistentFlags().Lookup("db")); err != nil {
		fmt.Fprintf(os.Stderr, "Error binding db flag: %v\n", err)
	}
	if err := viper.BindPFlag("addr", rootCmd.PersistentFlags().Lookup("addr")); err != nil {
		fmt.Fprintf(os.Stderr, "Error binding addr flag: %v\n", err)
	}
}

func initConfig() {
	if cfgFile != "" {
		viper.SetConfigFile(cfgFile)
	} else {
		home, err := os.UserHomeDir()
		cobra.CheckErr(err)

		viper.AddConfigPath(home)
		viper.SetConfigType("yaml")
		viper.SetConfigName(".mcias")
	}

	viper.AutomaticEnv()

	if err := viper.ReadInConfig(); err == nil {
		fmt.Fprintln(os.Stderr, "Using config file:", viper.ConfigFileUsed())
	}
}
